/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

#include <bootscript.h>
#include <common.h>
#include <mem_regions.h>
#include <watchdog.h>
#include <asm/io.h>
#include <asm/mach-types.h>
#include <asm/memprot.h>
#include <asm/arch/bcm4760.h>
#include <asm/arch/otp.h>

extern int __mmu_table_start; /* Defined in board/<board>/u-boot.lds */

DECLARE_GLOBAL_DATA_PTR;

/* Description of areas of OTP memory */
struct otp_area otp_areas[] = {
	/* Manufacturing data */
	{
		.mode	= OTP_ACCESS_MAN,
		.lower	= 0,
		.upper	= 3,
	},
	/* Configuration / security area */
	{
		.mode	= OTP_ACCESS_CON,
		.lower	= 4,
		.upper	= 69,
	},
	/* Unused / hardware control */
	{
		.mode	= OTP_ACCESS_RAM,
		.lower	= 70,
		.upper	= 95,
	},
	/* Sentinel */
	{
		.lower	= OTP_AREA_SENTINEL,
		.upper	= OTP_AREA_SENTINEL,
	}
};

/*******************************************************
 * delay - spinning delay before udelay works
 ******************************************************/
static inline void delay (unsigned long loops)
{
	__asm__ volatile ("1:\n" "subs %0, %1, #1\n"
		"bne 1b":"=r" (loops):"0" (loops));
}

static inline void bbl_wait(void)
{
        volatile u32 *io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);
	volatile u32 value;

        do
        {
                value = *io_ctl_addr;
        } while (value & 0x01 );
}

#define BBL_SW_REG3	0x29

/* Machine ID is read by Stage II from I2C PROM and passed in BBL */
static int read_bbl_machid(void)
{
        volatile u32 * io_sts_addr = (u32 *)(RTC_R_BLKSTS_MEMADDR);
        volatile u32 * io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);
        volatile u32 val;

        bbl_wait();

        *io_ctl_addr = (0x0001 | (BBL_SW_REG3 & 0xff) <<8);
        bbl_wait();

	val = (*io_sts_addr & 0xffff);

        return val;
}


/* Stage II sets a flag in the MSB bit to indicate serial recovery */
static int read_bbl_recovery_bit(void)
{
        volatile u32 * io_sts_addr = (u32 *)(RTC_R_BLKSTS_MEMADDR);
        volatile u32 * io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);
        volatile u32 val;

        bbl_wait();

        *io_ctl_addr = (0x0001 | (BBL_SW_REG3 & 0xff) <<8);
        bbl_wait();

	val = (*io_sts_addr & 0x80000000);
	val >>= 31;

        return val;
}



/*****************************************
 * board_init -early hardware init
 *****************************************/
int board_init (void)
{
	u32 v;
	char ulong_str[16]; /* Needs to be as long as "0x12345678" + '\0' */
	int m = read_bbl_machid();
	int rec = read_bbl_recovery_bit();

	/* Fallback if I2C PROM / FDT mach ID invalid */
	gd->bd->bi_arch_number = MACH_TYPE_SIENA;

	if (m != 0 && m != ~0) {
		gd->bd->bi_arch_number = m;
	}

	gd->bd->bi_boot_params = CFG_ATAG_ADDR;		/* address of boot params */

	v = readl(CMU_R_CMU_CTL0_MEMADDR);
	if (v & 0x40) {
		setenv("sysboot_mode","watchdog");
	} else {
		setenv("sysboot_mode","cold");
	}

	/* Check which register is used to determine the boot device */
	if ((v | ~0xf) == ~0) {
		/* OPT_STRAP[10:11] is the reference */
		v = readl(CMU_R_OTP_STRAP_MEMADDR) >> 10;
	} /* Otherwise, CMU_CTL0[0:1] is the reference */

	switch (v & 0x3) {
		case 1:  setenv("bootdev", "1");
			 break;
		case 2:  setenv("bootdev", "2");
			 break;
		default: eprintf("Invalid boot device, defaulting to 0\n");
		case 0:  setenv("bootdev", "0");
	}

	/* if serial recovery needed */
	if (rec) {
		setenv("force_altboot", "yes");
	} else {
		setenv("force_altboot", "no");
	}

	/* Set up a 'transparent' mapping so we can use the D-cache */
	set_default_map((uint32_t) &__mmu_table_start);

	IMPORT_BOOTSCRIPT(preboot_plat, preboot_plat);
#ifdef CONFIG_DEBUG_BUILD
	IMPORT_BOOTSCRIPT(preboot_debug, preboot);
	IMPORT_BOOTSCRIPT(bootcmd_debug, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd_debug, altbootcmd);
	IMPORT_BOOTSCRIPT(preboot, preboot.default);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd.default);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd.default);
#else
	IMPORT_BOOTSCRIPT(preboot, preboot);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd);
#endif

	/* Set up the environment */
	sprintf(ulong_str, "%p", CFG_LOAD_ADDR);
	setenv("loadaddr", ulong_str);

	sprintf(ulong_str, "%p", CFG_FDT_ADDR);
	setenv("fdaddr", ulong_str);

	sprintf(ulong_str, "%#x", MEMADDR_KERN_LEN);
	setenv("kernel.maxsize", ulong_str);

	return 0;
}

/*******************************************************
 * Routine: misc_init_r
 * Description: 
 ********************************************************/
int misc_init_r (void)
{
	return(0);
}

/**********************************************
 * dram_init - sets uboots idea of sdram size
 **********************************************/

#define MEM_SIZE_MAGIC_NUM 0x28BCBEAF
#define MEM_SIZE_MAGIC_OFFSET 0x200

int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	unsigned long mem;
    unsigned long orig_value;
	unsigned long aliasAddress = PHYS_SDRAM_1 + MEM_SIZE_MAGIC_OFFSET;

	/* Figure out memory size by finding aliases */
    orig_value = *(unsigned long*)aliasAddress;
	*(unsigned long *)aliasAddress = MEM_SIZE_MAGIC_NUM;

    /*
     * Hunt for where the MAGIC is duplicated
     */
	for (mem = SZ_1M; mem < SZ_128M; mem += SZ_1M) {
		if (*(unsigned long *)((unsigned long)(aliasAddress) + mem) == 
		    *(unsigned long *)(aliasAddress))
			break;
	}
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = mem;

    /* Restore alias location*/
    *(unsigned long*)aliasAddress = orig_value;

	return 0;
}

/*****************************************************************
 * peripheral_enable - enable the clks & power for perifs
 ******************************************************************/
void peripheral_enable(void)
{
#ifdef CFG_NS16550
#endif
}

