/*
 * Broadcom BCM4760 One Time Programmable Memory ([L]OTP) driver
 *
 * Copyright (C) 2010 TomTom International B.V.
 * Author: Martin Jackson <martin.jackson@tomtom.com>
 * 
 ************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 ************************************************************************
 *
 * The OTP is inherently unreliable. There are supposedly 2 modes of
 * failure of the OTP, according to Broadcom:
 *
 * 1) At manufacture, before programming, some bits are stuck permanantly
 *    at '1'
 * 2) During programming, some bits that are programmed as '1' will
 *    stick at '0' permanently
 **/

/* #define DEBUG */

#include <common.h>
#include <mem_regions.h>
#include <sdhci.h>
#ifdef CONFIG_TT_HEADER
# include <tt_header.h>
#endif
#include <asm/bitops.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <asm/arch/bcm4760_reg.h>
#include <asm/arch/otp.h>

#define OTP_TIMEOUT	0x10000
#define OTP_RETRIES	10

/* Base CTRL register setting for all OTP commands */
#define OTP_CTRL_BASE_VAL 0x00030800	/* Default VSEL   (0x00)
					 * Default PCOUNT (0x04)
					 * Default PYB    (0x00)
					 * Default SADBYP (0x00)
					 * Default TM     (0x04)
					 * CONTINUE_ON_FAIL is SET
					 * LOTP_DEBUG_MODE is UNSET
					 */

/* OTP block state */
static int otp_initflags;	/* Flags to initialize controller */

struct otp_request {
	otp_cmd_t cmd;		/* Command type */
	otp_addr_t addr;	/* Line number for OTP operation */
	otp_line_t data;	/* read input data / write output data */
	int flags;
};

#define debug_req(r) \
	debug("[" #r "]:\n" \
	       "\tcmd   = [%#x]\n" \
	       "\taddr  = [%#x]\n" \
	       "\tdata  = [%#x]\n", \
	       r->cmd, r->addr, r->data)

#define otpreg_read(addr) readl(addr)
#define otpreg_write(val, addr) writel(val, addr)

#define otp_err(s,args...)  eprintf("OTP ERROR: "   s, ##args);
#define otp_warn(s,args...) eprintf("OTP WARNING: " s, ##args);

static inline int otp_clear_status(void)
{
	uint32_t r = otpreg_read(OTP_R_LOTP_STAT_MEMADDR);
	otpreg_write(OTP_F_LOTP_STAT_ECC_COR_MASK |
		     OTP_F_LOTP_STAT_CMD_DONE_MASK,
		     OTP_R_LOTP_STAT_MEMADDR);
	return r;
}

static inline int otpreg_read_retry(uint32_t *reg, uint32_t mask,
				      uint32_t *val)
{
	int i;

	for(i=0; i<OTP_TIMEOUT; i++) {
		if ((*val = otpreg_read(reg)) & mask)
			return 0;
	}
	return -OTP_ETOUT;
}

#define otp_reinit(void) otp_init(otp_initflags)

static int otp_check_area(otp_addr_t addr)
{
	int i;

	for(i=0; !(otp_areas[i].lower == OTP_AREA_SENTINEL &&
		   otp_areas[i].upper == OTP_AREA_SENTINEL);
	    i++) {
		if (addr >= otp_areas[i].lower &&
		    addr <= otp_areas[i].upper) {
			return otp_areas[i].mode;
		}
	}
	return -OTP_EADDR; /* addr not part of otp_areas */
}

static int otp_start_command(struct otp_request *req, uint32_t ctrlval)
{
	uint32_t _t;
	int r,y;

	otpreg_write(req->addr<<OTP_F_LOTP_ADDR_R,
		     OTP_R_LOTP_ADDR_MEMADDR);

	/* Make sure the command starts successfully */
	otpreg_write(ctrlval, OTP_R_LOTP_CTRL_MEMADDR);
	udelay(10); /* A guess */
	for(y=0; y<OTP_RETRIES; y++) {
		if (otpreg_read(OTP_R_LOTP_STAT_MEMADDR) &
		    OTP_F_LOTP_STAT_BUSY_MASK) {
			break;
		}
		otp_err("READ command not started, retrying...\n");
		otp_clear_status();
		udelay(50<<y);
	}
	if (y == OTP_RETRIES && req->cmd == OTP_CMD_INIT) {
		otp_err("OTP block failure, system reset required.\n");
		return -OTP_EINIT;
	} else if (y == OTP_RETRIES) {
		return -OTP_ELOCKUP;
	}

	/* Wait for command to complete */
	r = otpreg_read_retry(OTP_R_LOTP_STAT_MEMADDR,
			      OTP_F_LOTP_STAT_CMD_DONE_MASK,
			      &_t);
	if (r) return r;

	if (req->cmd == OTP_CMD_READ) {
		r = otpreg_read_retry(OTP_R_LOTP_STAT_MEMADDR,
				      OTP_F_LOTP_STAT_DATA_READY_MASK,
				      &_t);

		if (r) return r;

		udelay(10); /* Recommended in reference implementation */
		r = otp_clear_status();

		req->data = otpreg_read(OTP_R_LOTP_RDATA_MEMADDR);
		if (req->flags & OTP_USE_ECC) {
			if (r & OTP_F_LOTP_STAT_ECC_COR_MASK) {
				otp_err("ECC correction occurred\n");
			} else if (r & OTP_F_LOTP_STAT_FAIL_MASK) {
				otp_err("Uncorrectable read error\n");
				return -OTP_EFAIL;
			}
			/* Remove ECC / FAIL bits from data word */
			req->data &= OTP_F_LOTP_RDATA_DOUT_MASK;
		}
	} else {
		r = otp_clear_status();
		/* Ignore FAIL status */
	}

	return 0;
}

static int otp_do_command(struct otp_request *req)
{
	uint32_t ctrlval = OTP_CTRL_BASE_VAL | OTP_CMD_READ;
	int r, retry;

	debug("Doing OTP command...\n");
	debug_req(req);

	/* Set up control register */
	switch (req->cmd) {
	case OTP_CMD_PROGRAM_BIT:
	case OTP_CMD_PROGRAM_LOCK_BIT:
	case OTP_CMD_PROGRAM_WORD:
		if (req->flags & OTP_USE_ECC) {
			__set_bit(OTP_F_LOTP_CTRL_ECC_WREN_R, &ctrlval);
			/* Blank out FAIL, ECC bits */
			req->data &= OTP_F_LOTP_WDATA_DIN_MASK;
		}
		otpreg_write(req->data, OTP_R_LOTP_WDATA_MEMADDR);
		r = otp_check_area(req->addr);
		if (r<0) {
			otp_err("Invalid OTP line address\n");
			return -OTP_EADDR;
		}
		ctrlval |= r << OTP_F_LOTP_CTRL_ACCESS_MODE_R;
		break;
	case OTP_CMD_READ:
		if (req->flags & OTP_USE_ECC) {
			__set_bit(OTP_F_LOTP_CTRL_ENECC_R, &ctrlval);
		}
		r = otp_check_area(req->addr);
		if (r<0) {
			otp_err("Invalid OTP line address\n");
			return -OTP_EADDR;
		}
		break;
	default:
		;
	}

	for(retry=0; retry<OTP_RETRIES; retry++) {
		r = otp_start_command(req, ctrlval);
		if (r == 0) { /* Success */
			debug("Reading OTP line complete\n");
			debug_req(req);
			return 0;
		} else if (r == -OTP_EINIT) {
			otp_err("Initialization error.\n");
		} else {
			otp_err("Resetting controller.\n");
			otp_reinit();
		} 
		debug("Retrying command...");
	}

	/* Error occurred */
	if (r == -OTP_EFAIL) {
		return r;
	} else {
		otp_err("OTP block failure, system reset required.\n");
		return -OTP_ELOCKUP;
	}

}

int otp_send_initcmd(void)
{
	struct otp_request req = {
		.cmd   = OTP_CMD_INIT,
		.addr  = 0xdeadbeef,
		.data  = 0xdeadbeef,
		.flags = 0x0,
	};

	return otp_do_command(&req);
}

int otp_read(otp_addr_t addr, otp_line_t *rdata, int flags)
{
	int r;

	struct otp_request req = {
		.cmd   = OTP_CMD_READ,
		.addr  = addr,
		.data  = 0xdeadbeef,
		.flags = flags,
	};

	r = otp_do_command(&req);
	*rdata = req.data;

	return r;
}

#if 0 /* Not yet implemented */
int otp_bit_write(otp_addr_t addr, otp_line_t wdata, int flags)
{
	otpreg_write(wdata, OTP_R_LOTP_WDATA_MEMADDR);
	otpreg_write(addr, OTP_R_LOTP_ADDR_MEMADDR);
	//otp_control_write(addr, OTP_CMD_PROGRAM_BIT);
	//complete_cmd();

	/* MAJA - Why no second write here? */

	return 0;
}
#endif

#if 0 /* Not yet implemented */
int otp_bit_write_lock(otp_addr_t addr, otp_line_t wdata, int flags)
{
	otpreg_write(wdata, OTP_R_LOTP_WDATA_MEMADDR);
	otpreg_write(addr, OTP_R_LOTP_ADDR_MEMADDR);
	//otp_control_write(addr, OTP_CMD_LOCK);
	//complete_cmd();

	return 0;
}
#endif

#if 0 /* Not yet implemented */
int otp_write(otp_addr_t addr, otp_line_t *wdata, int flags)
{	
	otpreg_write(*wdata, OTP_R_LOTP_WDATA_MEMADDR);
	otpreg_write(addr, OTP_R_LOTP_ADDR_MEMADDR);
	//otp_control_write(addr, OTP_CMD_PROGRAM_WORD);
	//complete_cmd();

	/* Do it again for the second fuse */
	otpreg_write(*wdata, OTP_R_LOTP_WDATA_MEMADDR);
	otpreg_write(addr, OTP_R_LOTP_ADDR_MEMADDR);
	//otp_control_write(addr, OTP_CMD_PROGRAM_WORD+OTP_F_LOTP_CTRL_PROGSEL_MASK);
	//complete_cmd();

	return 0;
}
#else
int otp_write(otp_addr_t addr, otp_line_t *wdata, int flags)
{
	otp_err("\"otp_write\" Not yet implemented\n");
	return -OTP_ENOAPI;
}
#endif

/* Disables 8 lines per bit in SECURE1 */
int otp_line_disable(otp_addr_t start, uint num)
{
	/* Truncate if necessary */
	uint i,n = ((start+num)>>3) + !!((start+num)&0x7) - (start>>3);
	uint32_t t;

	if (otp_check_area(start)<0) {
		otp_err("Tried to disable invalid OTP line %d\n", start);
		return -OTP_EADDR;
	} else if (otp_check_area(start+num)<0) {
		otp_err("Tried to disable invalid OTP line %d\n",
			start+num);
		return -OTP_EADDR;
	}

	t = otpreg_read(OTP_R_LOTP_SECURE1_MEMADDR);
	debug("SECURE1 = %#x\n", t);
	for(i=0; i<n; i++) {
		debug("Disabling row %d to %d of OTP\n", start + (i<<3),
		      start + (i<<3) + 7);
		__set_bit((start>>3) + i, &t);
	}
	debug("Setting SECURE1 = %#x\n", t);
	otpreg_write(t, OTP_R_LOTP_SECURE1_MEMADDR);

#ifdef DEBUG
	t = otpreg_read(OTP_R_LOTP_SECURE0_MEMADDR);
	debug("SECURE0 = %#x\n", t);

	t = otpreg_read(OTP_R_LOTP_SECURE1_MEMADDR);
	debug("SECURE1 = %#x\n", t);
#endif
		
	return 0;
}

/* Wait for hardware to check secure area CRC on startup */
static int crc_check(void)
{
	uint32_t _t;

	debug("Waiting for CRC check...\n");
	while (otpreg_read_retry(OTP_R_OTP_CONTROL_MEMADDR,
				 OTP_F_LOTP_CRC_DONE_MASK,
				 &_t) == -OTP_ETOUT) {
		otp_err("CRC check timed out\n");
	}

	if (!(otpreg_read(OTP_R_OTP_CONTROL_MEMADDR) &
	      OTP_F_LOTP_UNPROG_MASK) &&
	    !(otpreg_read(OTP_R_OTP_CONTROL_MEMADDR) &
	      OTP_F_LOTP_PROG_SUCCESS_MASK))
	{
		/* Ignore PROG_UNSUCCESS bit */
		otp_err("CRC check FAILED!\n");
		return -OTP_ECRC;
	}

	debug("CRC check finished (CRC OK).\n");
	return 0;
}

/* Assumes that OTP block is enabled in CMU */
int otp_init(int flags)
{
	uint32_t t;
	int r;

	debug("Resetting OTP controller\n");
	t = otpreg_read(OTP_R_OTP_CONTROL_MEMADDR);
	__set_bit(OTP_F_RST_R, &t);
	otpreg_write(t, OTP_R_OTP_CONTROL_MEMADDR);
	udelay(30);
	t = otpreg_read(OTP_R_OTP_CONTROL_MEMADDR);
	__clear_bit(OTP_F_RST_R, &t);
	otpreg_write(t, OTP_R_OTP_CONTROL_MEMADDR);

	if (crc_check()) {
		return -OTP_EINIT;
	}

	/* Set up OTP_CONTROL register
	 * Leave OTP block in default (slowest) clock divider setting
	 */
	debug("Initializing OTP controller...\n");
	t = otpreg_read(OTP_R_OTP_CONTROL_MEMADDR);
	__set_bit(OTP_F_LOTP_CPU_MODE_EN_R, &t);
	__set_bit(OTP_F_LOTP_ENABLE_R, &t);
	__clear_bit(OTP_F_RST_R, &t);
	otpreg_write(t, OTP_R_OTP_CONTROL_MEMADDR);

	/* Disable LOTP correctable counter feature & reset count */
	otpreg_write(0x0, OTP_R_LOTP_ECC_CNT_MEMADDR);

	otp_clear_status();
	otp_initflags = flags;

	if ((r = otp_send_initcmd())) {
		return -OTP_EINIT;
	}

	if (otpreg_read(OTP_R_OTP_CONTROL_MEMADDR) &
	    OTP_F_LOTP_UNPROG_MASK) {
		/* Can still use the OTP, but can't trust its content */
		/* otp_warn("OTP has not been programmed!\n"); */
		return -OTP_ENOPROG;
	}

	debug("OTP controller initialized OK.\n");
	return 0;
}

/**
 * Read an integer in priority encoded redundant format from OTP
 *
 * Each integer consists of priority encoded 'logical' bits (i.e. 3 =>
 * b00...0111, 2 => b00...0011 etc.). Each 'logical' bit is encoded
 * redundantly with 3 'physical' bits. The logical bit is '1' if 2 or
 * more of the 3 physical bits are '1' and is considered to be '0'
 * otherwise. In a 32-bit word, it is possible to encode integers from 0
 * up to 10. The leftmost 2 of the 32 bits are ignored. A 32 bit integer
 * is little endian.
 *
 * The leftmost logical bit is taken as definitive.
 *
 * For example:
 *  b0000,0000,1000,0010,0000,1110,1111,0111 = 4  [ f7 0e 82 00 ]
 *  b1110,0000,0000,0000,0000,0000,0010,1111 = 2  [ 2f 00 00 e0 ]
 *  b0011,0000,0000,0000,0000,0000,0000,0000 = 10 [ 00 00 00 30 ]
 **/
int otp_read_bricknum(otp_addr_t addr)
{
	int i, r;
	uint32_t mask1 = 0x20000000;
	uint32_t mask2 = 0x18000000;
	uint32_t data;

	r = otp_read(addr, (otp_line_t *)&data, OTP_NO_ECC);
	if (r) {
		return r;
	}

	for(i=10; mask1 != 0x0; mask1>>=3, mask2>>=3, i--) {
		if ((mask1 & data && mask2 & data) ||
		    (!(mask1 & data) && !~(~mask2 & data))) {
			break;
		}
	}
	return i;
}

#ifdef CONFIG_TT_HEADER

/* Number of blocks to kill at start of MoviNand if check fails, must
 * kill stage II
 */
#define KILL_BLOCKS 64

int check_bricknum(void)
{
	otp_line_t otpbnum;
	int hdrbnum = ((struct tt_header *)
		(TEXT_BASE + TT_HEADER_OFFSET))->brick;

	debug("Checking brick number of U-Boot ...\n");

	/* stderr not yet initialized, use printf for error msgs */

#define do_read_bricknum(n) ({						\
		otp_line_t b = otp_read_bricknum(n);			\
		if (b < 0) {						\
			printf("ERROR: failed to get bricknum "		\
			       "(OTP row %d)\n", n);			\
			return 1;					\
		}							\
		b;							\
	})

	otpbnum = do_read_bricknum(OTP_UBOOT_BRICKNUM);

	if (otpbnum <= hdrbnum) {
		debug("Brick number check OK\n");
	} else {
		char *bootdev_str;
		ulong bootdev;
		printf("ERROR: Brick number check FAILED (read %d from "
		       "OTP, expected %d)\n", otpbnum, hdrbnum);
#ifdef CONFIG_DEBUG_BUILD
		return 0;
#else
		bootdev_str = getenv("bootdev");
		if (bootdev_str == NULL) {
			return 1; /* HANG */
		}
		bootdev = simple_strtoul(bootdev_str, NULL, 10);
		printf("Prepare for serial recovery: delete stage II\n");
		sdhci_init(bootdev);

		memset(MEMADDR_FDT_START, 0, KILL_BLOCKS*512);
		sdhci_write(bootdev, 2, KILL_BLOCKS, MEMADDR_FDT_START);
		return 1;
#endif
	}

#define setenv_bricknum(n, e) {				\
		otp_line_t t = do_read_bricknum(n);	\
		/* 0 < t <= 10, no buffer overflow */	\
		char s[3];				\
		sprintf(s, "%d", t);			\
		setenv(e, s);				\
	}

	setenv_bricknum(OTP_KERN_BRICKNUM, "kernel.bricknum");
	setenv_bricknum(OTP_RESCUE_BRICKNUM, "kernel.bricknum.rescue");
	return 0;
}

#endif	/* CONFIG_TT_HEADER */

