/*
 * elgamalenc.c - encrypt the box key using Elgamal encryption algorithm
 *
 * 2009-11-03 Ard Biesheuvel <ard.biesheuvel@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <mpi.h>
#include <asm/errno.h>
#include <unrandom.h>

#include "elgamal_pubkey.h"

#define L	(sizeof(P)*8)

static struct _mpi const
	p = { .alloced = L/BITS_PER_MPI_LIMB, .nlimbs = L/BITS_PER_MPI_LIMB, .nbits = L, .sign = 0, .flags = 0, .d = (mpi_limb_t*)P },
	g = { .alloced = L/BITS_PER_MPI_LIMB, .nlimbs = L/BITS_PER_MPI_LIMB, .nbits = L, .sign = 0, .flags = 0, .d = (mpi_limb_t*)G },
	y = { .alloced = L/BITS_PER_MPI_LIMB, .nlimbs = L/BITS_PER_MPI_LIMB, .nbits = L, .sign = 0, .flags = 0, .d = (mpi_limb_t*)Y };

static int print_mpi(unsigned char *out, MPI in)
{
	static char const tohex[] = "0123456789ABCDEF";
	int i;

	/* always produce the same number of hex digits */
	for ( i = L/BITS_PER_MPI_LIMB; i > in->nlimbs; i-- ) {
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
		*out++ = '0';
	}

	for ( i = in->nlimbs - 1; i >= 0; i-- )
	{
		*out++ = tohex[ (in->d[i] >> 28) & 0xf ];
		*out++ = tohex[ (in->d[i] >> 24) & 0xf ];
		*out++ = tohex[ (in->d[i] >> 20) & 0xf ];
		*out++ = tohex[ (in->d[i] >> 16) & 0xf ];
		*out++ = tohex[ (in->d[i] >> 12) & 0xf ];
		*out++ = tohex[ (in->d[i] >>  8) & 0xf ];
		*out++ = tohex[ (in->d[i] >>  4) & 0xf ];
		*out++ = tohex[ (in->d[i]      ) & 0xf ];
	}
	return 8*in->nlimbs;
}

static MPI generate_k(void )
{
	unsigned char buf[UNRANDOM_SIZE];

	MPI k 		= mpi_alloc(UNRANDOM_SIZE/BYTES_PER_MPI_LIMB);
	MPI temp	= mpi_alloc(mpi_get_nlimbs(&p));
	MPI p_1		= mpi_alloc(mpi_get_nlimbs(&p));

	UNRANDOM_GET(buf);

	mpi_sub_ui(p_1, (MPI)&p, 1);
	mpi_set_buffer(k, buf, UNRANDOM_SIZE, 0);

	for (;;) {
		if (mpi_gcd(temp, k, p_1))
			return k;
		mpi_add_ui(k, k, 1);
	}
	return NULL;
}

int elgamal_encrypt(unsigned char *ciphertext, unsigned char const *plaintext, int plaintext_size)
{
	MPI k, c, m;
	int l;

	k = generate_k();
	c = mpi_alloc( mpi_get_nlimbs(&p) );
	m = mpi_alloc(plaintext_size/BYTES_PER_MPI_LIMB);
	mpi_set_buffer(m, plaintext, plaintext_size, 0);

	/* calculate C1 and write to output */
	if (0 > mpi_powm( c, (MPI)&g, k, (MPI)&p ))	return -ENOMEM;

	l = print_mpi( ciphertext, c );

	/* calculate C2 and write to output */
	if (0 > mpi_powm( c, (MPI)&y, k, (MPI)&p ))	return -ENOMEM;
	if (0 > mpi_mulm( c, c, m, (MPI)&p ))		return -ENOMEM;

	print_mpi( ciphertext + l, c );

	return 0;		
}

