/*
 * Broadcom BCM4760 One Time Programmable Memory ([L]OTP) driver
 *
 * Copyright (C) 2010 TomTom International B.V.
 * Author: Martin Jackson <martin.jackson@tomtom.com>
 * 
 ************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 ************************************************************************
 */

#ifndef OTP_CPUAPI_H
#define OTP_CPUAPI_H

#include <common.h>
#include <asm/arch/bcm4760_reg.h>

#ifndef UINT_MAX
# define UINT_MAX (1<<(sizeof(uint)*8-1))
#endif

#define OTP_USE_ECC	0x00000001
#define OTP_NO_ECC	0x00000002

/* OTP Layout on TomTom devices */
#define OTP_UBOOT_BRICKNUM	((otp_line_t) 70)
#define OTP_KERN_BRICKNUM	((otp_line_t) 71)
#define OTP_RESCUE_BRICKNUM	((otp_line_t) 72)

enum otp_cmds {
	OTP_CMD_READ			= 0x0, 
	OTP_CMD_PROGRAM_BIT		= 0x1,
	OTP_CMD_PROGRAM_WORD		= 0x2,
	OTP_CMD_VERIFY			= 0x3,
	OTP_CMD_INIT			= 0x4,
	OTP_CMD_SET			= 0x5,
	OTP_CMD_RESET			= 0x6,
	OTP_CMD_OCST			= 0x7,
	OTP_CMD_PROGRAM_LOCK_BIT	= 0x8,
	OTP_CMD_PRESCREEN_TEST		= 0x9,
	OTP_CMD_PROGRAM_BIT_EXT		= 0xA,
	OTP_CMD_PROGRAM_WORD_EXT	= 0xB,
};
typedef enum otp_cmds otp_cmd_t;

typedef uint32_t otp_line_t;		/* OTP line */
typedef uint otp_addr_t;	/* Addr. space of 32 bit lines */
#define OTP_AREA_SENTINEL UINT_MAX

enum otp_access_mode {
	OTP_ACCESS_RAW	= 0x0,	/* RAW access */
	OTP_ACCESS_MAN	= 0x1,	/* Manufacturing area access */
	OTP_ACCESS_CON	= 0x2,	/* Configuration area access */
	OTP_ACCESS_RAM	= 0x3,	/* "RAM repair" (unused) area access */
};

struct otp_area {
	enum otp_access_mode mode;
	otp_addr_t lower;
	otp_addr_t upper;
};

/* Definition provided by the board code, all OTP areas described. */
extern struct otp_area otp_areas[];

int otp_write(otp_addr_t addr, otp_line_t *wdata, int flags);
int otp_bit_write(otp_addr_t addr, otp_line_t wdata, int flags);
int otp_bit_write_lock(otp_addr_t addr, otp_line_t wdata, int flags);
/* Check range */

/* OTP errors */
enum otp_err {
/* OTP errors, can be returned from public interface */
	OTP_EINIT = 1,	/* OTP contr. initialization failed */
	OTP_ELOCKUP,	/* OTP contr. locked up, system reset required */
	OTP_EFAIL,	/* Uncorrectable OTP line error */
	OTP_ENOPROG,	/* OTP has not been programmed */
	OTP_EADDR,	/* OTP line address is out-of-range */
	OTP_ENOAPI,	/* Function not yet implemented */

/* OTP internal errors, not returned from public interface */
	OTP_ETOUT,	/* Operation timed out */
	OTP_ECRC,	/* OTP CRC Check failed */
};

int otp_read(otp_addr_t addr, otp_line_t *rdata, int flags);
/* Disable reading of OTP line til full system reset */
int otp_line_disable(otp_addr_t start, uint num);
int otp_read_bricknum(otp_addr_t addr);
int otp_init(int flags);

#define LOTP_SEC_CFG_ROW_L	(8 << 5)
#define LOTP_MFG_KEYS_ROW_BOUND	(LOTP_SEC_CFG_ROW_L + (64-5)*32 - 1) 

/* Field: CFG_F_OTP_SCFG                                                                    */
/* Description: "OTP secure configuration bits, from OTP "                                  */
#define CFG_F_OTP_SCFG_R                                                                   0
#define CFG_F_OTP_SCFG_L                                                                   7
#define CFG_F_OTP_SCFG_SIZE                                                                8
#define CFG_F_OTP_SCFG_MASK                                                       0x000000ff
#define CFG_F_OTP_SCFG_RANGE                                                             7:0
#define CFG_F_OTP_SCFG_INIT                                                       0x00000000
#define CFG_F_OTP_SCFG_RWMODE                                                             RO
#define CFG_F_OTP_SCFG_GET(x) \
    (((x) & CFG_F_OTP_SCFG_MASK) >> \
            CFG_F_OTP_SCFG_R) \
/* Register: OTP_R_LOTP_ECC_CNT                                                             */
#define OTP_R_LOTP_ECC_CNT_MEMADDR                                                 0xC8018
#define OTP_R_LOTP_ECC_CNT_SEL                                                          0x18
#define OTP_R_LOTP_ECC_CNT_SIZE                                                           32
#define OTP_R_LOTP_ECC_CNT_MASK                                                   0xffffffff
#define OTP_R_LOTP_ECC_CNT_INIT                                                          0x0
#define OTP_R_LOTP_ECC_CNT_RWMODE                                                         RW
/* Field: OTP_F_ECC_CCNT                                                                    */
#define OTP_F_ECC_CCNT_R                                                                   0
#define OTP_F_ECC_CCNT_L                                                                  15
#define OTP_F_ECC_CCNT_SIZE                                                               16
#define OTP_F_ECC_CCNT_MASK                                                       0x0000ffff
#define OTP_F_ECC_CCNT_RANGE                                                            15:0
#define OTP_F_ECC_CCNT_INIT                                                       0x00000000
#define OTP_F_ECC_CCNT_RWMODE                                                            SRO
#define OTP_F_ECC_CCNT_GET(x) \
    (((x) & OTP_F_ECC_CCNT_MASK) >> \
            OTP_F_ECC_CCNT_R) \
/* Field: OTP_F_ECC_CTHRESH                                                                 */
#define OTP_F_ECC_CTHRESH_R                                                               16
#define OTP_F_ECC_CTHRESH_L                                                               31
#define OTP_F_ECC_CTHRESH_SIZE                                                            16
#define OTP_F_ECC_CTHRESH_MASK                                                    0xffff0000
#define OTP_F_ECC_CTHRESH_RANGE                                                        31:16
#define OTP_F_ECC_CTHRESH_INIT                                                    0x00000000
#define OTP_F_ECC_CTHRESH_RWMODE                                                          RW
#define OTP_F_ECC_CTHRESH_GET(x) \
    (((x) & OTP_F_ECC_CTHRESH_MASK) >> \
            OTP_F_ECC_CTHRESH_R) \
/* Register: OTP_R_LOTP_STAT                                                                */
#define OTP_R_LOTP_STAT_MEMADDR                                                    0xC8010
#define OTP_R_LOTP_STAT_SEL                                                             0x10
#define OTP_R_LOTP_STAT_SIZE                                                              32
#define OTP_R_LOTP_STAT_MASK                                                      0xffffffff
#define OTP_R_LOTP_STAT_INIT                                                             0x0
#define OTP_R_LOTP_STAT_RWMODE                                                           SRO
/* Field: OTP_F_LOTP_STAT_BUSY                                                              */
#define OTP_F_LOTP_STAT_BUSY_R                                                             3
#define OTP_F_LOTP_STAT_BUSY_L                                                             3
#define OTP_F_LOTP_STAT_BUSY_SIZE                                                          1
#define OTP_F_LOTP_STAT_BUSY_MASK                                                 0x00000008
#define OTP_F_LOTP_STAT_BUSY_RANGE                                                       3:3
#define OTP_F_LOTP_STAT_BUSY_INIT                                                 0x00000000
#define OTP_F_LOTP_STAT_BUSY_RWMODE                                                      SRO
#define OTP_F_LOTP_STAT_BUSY_GET(x) \
    (((x) & OTP_F_LOTP_STAT_BUSY_MASK) >> \
            OTP_F_LOTP_STAT_BUSY_R) \
/* Field: OTP_F_LOTP_STAT_FAIL                                                              */
#define OTP_F_LOTP_STAT_FAIL_R                                                             4
#define OTP_F_LOTP_STAT_FAIL_L                                                             4
#define OTP_F_LOTP_STAT_FAIL_SIZE                                                          1
#define OTP_F_LOTP_STAT_FAIL_MASK                                                 0x00000010
#define OTP_F_LOTP_STAT_FAIL_RANGE                                                       4:4
#define OTP_F_LOTP_STAT_FAIL_INIT                                                 0x00000000
#define OTP_F_LOTP_STAT_FAIL_RWMODE                                                      SRO
#define OTP_F_LOTP_STAT_FAIL_GET(x) \
    (((x) & OTP_F_LOTP_STAT_FAIL_MASK) >> \
            OTP_F_LOTP_STAT_FAIL_R) \
/* Field: OTP_F_LOTP_STAT_INVALID_PROG_REQ                                                  */
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_R                                                 5
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_L                                                 5
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_SIZE                                              1
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_MASK                                     0x00000020
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_RANGE                                           5:5
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_INIT                                     0x00000000
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_RWMODE                                          SRO
#define OTP_F_LOTP_STAT_INVALID_PROG_REQ_GET(x) \
    (((x) & OTP_F_LOTP_STAT_INVALID_PROG_REQ_MASK) >> \
            OTP_F_LOTP_STAT_INVALID_PROG_REQ_R) \
/* Field: OTP_F_LOTP_STAT_DOUT                                                              */
#define OTP_F_LOTP_STAT_DOUT_R                                                             2
#define OTP_F_LOTP_STAT_DOUT_L                                                             2
#define OTP_F_LOTP_STAT_DOUT_SIZE                                                          1
#define OTP_F_LOTP_STAT_DOUT_MASK                                                 0x00000004
#define OTP_F_LOTP_STAT_DOUT_RANGE                                                       2:2
#define OTP_F_LOTP_STAT_DOUT_INIT                                                 0x00000000
#define OTP_F_LOTP_STAT_DOUT_RWMODE                                                      SRO
#define OTP_F_LOTP_STAT_DOUT_GET(x) \
    (((x) & OTP_F_LOTP_STAT_DOUT_MASK) >> \
            OTP_F_LOTP_STAT_DOUT_R) \
/* Field: OTP_F_LOTP_STAT_ECC_COR                                                           */
#define OTP_F_LOTP_STAT_ECC_COR_R                                                          8
#define OTP_F_LOTP_STAT_ECC_COR_L                                                          8
#define OTP_F_LOTP_STAT_ECC_COR_SIZE                                                       1
#define OTP_F_LOTP_STAT_ECC_COR_MASK                                              0x00000100
#define OTP_F_LOTP_STAT_ECC_COR_RANGE                                                    8:8
#define OTP_F_LOTP_STAT_ECC_COR_INIT                                              0x00000000
#define OTP_F_LOTP_STAT_ECC_COR_RWMODE                                                  SW1C
#define OTP_F_LOTP_STAT_ECC_COR_GET(x) \
    (((x) & OTP_F_LOTP_STAT_ECC_COR_MASK) >> \
            OTP_F_LOTP_STAT_ECC_COR_R) \
/* Field: OTP_F_LOTP_STAT_CMD_DONE                                                          */
#define OTP_F_LOTP_STAT_CMD_DONE_R                                                         0
#define OTP_F_LOTP_STAT_CMD_DONE_L                                                         0
#define OTP_F_LOTP_STAT_CMD_DONE_SIZE                                                      1
#define OTP_F_LOTP_STAT_CMD_DONE_MASK                                             0x00000001
#define OTP_F_LOTP_STAT_CMD_DONE_RANGE                                                   0:0
#define OTP_F_LOTP_STAT_CMD_DONE_INIT                                             0x00000000
#define OTP_F_LOTP_STAT_CMD_DONE_RWMODE                                                 SW1C
#define OTP_F_LOTP_STAT_CMD_DONE_GET(x) \
    (((x) & OTP_F_LOTP_STAT_CMD_DONE_MASK) >> \
            OTP_F_LOTP_STAT_CMD_DONE_R) \
/* Field: OTP_F_OTP_BISR_AUTO_LOAD_EN                                                       */
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_R                                                      9
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_L                                                      9
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_SIZE                                                   1
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_MASK                                          0x00000200
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_RANGE                                                9:9
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_INIT                                          0x00000000
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_RWMODE                                               SRO
#define OTP_F_OTP_BISR_AUTO_LOAD_EN_GET(x) \
    (((x) & OTP_F_OTP_BISR_AUTO_LOAD_EN_MASK) >> \
            OTP_F_OTP_BISR_AUTO_LOAD_EN_R) \
/* Field: OTP_F_OTP_BISR_CHECKSUM_ERR                                                       */
#define OTP_F_OTP_BISR_CHECKSUM_ERR_R                                                     10
#define OTP_F_OTP_BISR_CHECKSUM_ERR_L                                                     10
#define OTP_F_OTP_BISR_CHECKSUM_ERR_SIZE                                                   1
#define OTP_F_OTP_BISR_CHECKSUM_ERR_MASK                                          0x00000400
#define OTP_F_OTP_BISR_CHECKSUM_ERR_RANGE                                              10:10
#define OTP_F_OTP_BISR_CHECKSUM_ERR_INIT                                          0x00000000
#define OTP_F_OTP_BISR_CHECKSUM_ERR_RWMODE                                               SRO
#define OTP_F_OTP_BISR_CHECKSUM_ERR_GET(x) \
    (((x) & OTP_F_OTP_BISR_CHECKSUM_ERR_MASK) >> \
            OTP_F_OTP_BISR_CHECKSUM_ERR_R) \
/* Field: OTP_F_OTP_BISR_GO                                                                 */
#define OTP_F_OTP_BISR_GO_R                                                               11
#define OTP_F_OTP_BISR_GO_L                                                               11
#define OTP_F_OTP_BISR_GO_SIZE                                                             1
#define OTP_F_OTP_BISR_GO_MASK                                                    0x00000800
#define OTP_F_OTP_BISR_GO_RANGE                                                        11:11
#define OTP_F_OTP_BISR_GO_INIT                                                    0x00000000
#define OTP_F_OTP_BISR_GO_RWMODE                                                         SRO
#define OTP_F_OTP_BISR_GO_GET(x) \
    (((x) & OTP_F_OTP_BISR_GO_MASK) >> \
            OTP_F_OTP_BISR_GO_R) \
/* Field: OTP_F_LOTP_STAT_DATA_READY                                                        */
#define OTP_F_LOTP_STAT_DATA_READY_R                                                       1
#define OTP_F_LOTP_STAT_DATA_READY_L                                                       1
#define OTP_F_LOTP_STAT_DATA_READY_SIZE                                                    1
#define OTP_F_LOTP_STAT_DATA_READY_MASK                                           0x00000002
#define OTP_F_LOTP_STAT_DATA_READY_RANGE                                                 1:1
#define OTP_F_LOTP_STAT_DATA_READY_INIT                                           0x00000000
#define OTP_F_LOTP_STAT_DATA_READY_RWMODE                                                SRO
#define OTP_F_LOTP_STAT_DATA_READY_GET(x) \
    (((x) & OTP_F_LOTP_STAT_DATA_READY_MASK) >> \
            OTP_F_LOTP_STAT_DATA_READY_R) \
/* Field: OTP_F_OTP_BISR_DONE                                                               */
#define OTP_F_OTP_BISR_DONE_R                                                             12
#define OTP_F_OTP_BISR_DONE_L                                                             12
#define OTP_F_OTP_BISR_DONE_SIZE                                                           1
#define OTP_F_OTP_BISR_DONE_MASK                                                  0x00001000
#define OTP_F_OTP_BISR_DONE_RANGE                                                      12:12
#define OTP_F_OTP_BISR_DONE_INIT                                                  0x00000000
#define OTP_F_OTP_BISR_DONE_RWMODE                                                       SRO
#define OTP_F_OTP_BISR_DONE_GET(x) \
    (((x) & OTP_F_OTP_BISR_DONE_MASK) >> \
            OTP_F_OTP_BISR_DONE_R) \
/* Register: OTP_R_LOTP_WDATA                                                               */
#define OTP_R_LOTP_WDATA_MEMADDR                                                   0xC8004
#define OTP_R_LOTP_WDATA_SEL                                                             0x4
#define OTP_R_LOTP_WDATA_SIZE                                                             32
#define OTP_R_LOTP_WDATA_MASK                                                     0xffffffff
#define OTP_R_LOTP_WDATA_INIT                                                            0x0
#define OTP_R_LOTP_WDATA_RWMODE                                                           RW
/* Field: OTP_F_LOTP_WDATA_FAIL                                                             */
#define OTP_F_LOTP_WDATA_FAIL_R                                                           29
#define OTP_F_LOTP_WDATA_FAIL_L                                                           31
#define OTP_F_LOTP_WDATA_FAIL_SIZE                                                         3
#define OTP_F_LOTP_WDATA_FAIL_MASK                                                0xe0000000
#define OTP_F_LOTP_WDATA_FAIL_RANGE                                                    31:29
#define OTP_F_LOTP_WDATA_FAIL_INIT                                                0x00000000
#define OTP_F_LOTP_WDATA_FAIL_RWMODE                                                      RW
#define OTP_F_LOTP_WDATA_FAIL_GET(x) \
    (((x) & OTP_F_LOTP_WDATA_FAIL_MASK) >> \
            OTP_F_LOTP_WDATA_FAIL_R) \
/* Field: OTP_F_LOTP_WDATA_DIN                                                              */
#define OTP_F_LOTP_WDATA_DIN_R                                                             0
#define OTP_F_LOTP_WDATA_DIN_L                                                            23
#define OTP_F_LOTP_WDATA_DIN_SIZE                                                         24
#define OTP_F_LOTP_WDATA_DIN_MASK                                                 0x00ffffff
#define OTP_F_LOTP_WDATA_DIN_RANGE                                                      23:0
#define OTP_F_LOTP_WDATA_DIN_INIT                                                 0x00000000
#define OTP_F_LOTP_WDATA_DIN_RWMODE                                                       RW
#define OTP_F_LOTP_WDATA_DIN_GET(x) \
    (((x) & OTP_F_LOTP_WDATA_DIN_MASK) >> \
            OTP_F_LOTP_WDATA_DIN_R) \
/* Field: OTP_F_LOTP_WDATA_ECC                                                              */
#define OTP_F_LOTP_WDATA_ECC_R                                                            24
#define OTP_F_LOTP_WDATA_ECC_L                                                            28
#define OTP_F_LOTP_WDATA_ECC_SIZE                                                          5
#define OTP_F_LOTP_WDATA_ECC_MASK                                                 0x1f000000
#define OTP_F_LOTP_WDATA_ECC_RANGE                                                     28:24
#define OTP_F_LOTP_WDATA_ECC_INIT                                                 0x00000000
#define OTP_F_LOTP_WDATA_ECC_RWMODE                                                       RW
#define OTP_F_LOTP_WDATA_ECC_GET(x) \
    (((x) & OTP_F_LOTP_WDATA_ECC_MASK) >> \
            OTP_F_LOTP_WDATA_ECC_R) \
/* Register: OTP_R_OTP_CONTROL                                                              */
#define OTP_R_OTP_CONTROL_MEMADDR                                                  0xC8000
#define OTP_R_OTP_CONTROL_SEL                                                            0x0
#define OTP_R_OTP_CONTROL_SIZE                                                            32
#define OTP_R_OTP_CONTROL_MASK                                                    0xffffffff
#define OTP_R_OTP_CONTROL_INIT                                                      0x600000
#define OTP_R_OTP_CONTROL_RWMODE                                                          RW
/* Field: OTP_F_RST                                                                         */
#define OTP_F_RST_R                                                                       14
#define OTP_F_RST_L                                                                       14
#define OTP_F_RST_SIZE                                                                     1
#define OTP_F_RST_MASK                                                            0x00004000
#define OTP_F_RST_RANGE                                                                14:14
#define OTP_F_RST_INIT                                                            0x00000000
#define OTP_F_RST_RWMODE                                                                  RW
#define OTP_F_RST_GET(x) \
    (((x) & OTP_F_RST_MASK) >> \
            OTP_F_RST_R) \
/* Field: OTP_F_LOTP_UNPROG                                                                 */
#define OTP_F_LOTP_UNPROG_R                                                                1
#define OTP_F_LOTP_UNPROG_L                                                                1
#define OTP_F_LOTP_UNPROG_SIZE                                                             1
#define OTP_F_LOTP_UNPROG_MASK                                                    0x00000002
#define OTP_F_LOTP_UNPROG_RANGE                                                          1:1
#define OTP_F_LOTP_UNPROG_INIT                                                    0x00000000
#define OTP_F_LOTP_UNPROG_RWMODE                                                         SRO
#define OTP_F_LOTP_UNPROG_GET(x) \
    (((x) & OTP_F_LOTP_UNPROG_MASK) >> \
            OTP_F_LOTP_UNPROG_R) \
/* Field: OTP_F_LOTP_CRC_DONE                                                               */
#define OTP_F_LOTP_CRC_DONE_R                                                              0
#define OTP_F_LOTP_CRC_DONE_L                                                              0
#define OTP_F_LOTP_CRC_DONE_SIZE                                                           1
#define OTP_F_LOTP_CRC_DONE_MASK                                                  0x00000001
#define OTP_F_LOTP_CRC_DONE_RANGE                                                        0:0
#define OTP_F_LOTP_CRC_DONE_INIT                                                  0x00000000
#define OTP_F_LOTP_CRC_DONE_RWMODE                                                       SRO
#define OTP_F_LOTP_CRC_DONE_GET(x) \
    (((x) & OTP_F_LOTP_CRC_DONE_MASK) >> \
            OTP_F_LOTP_CRC_DONE_R) \
/* Field: OTP_F_LOTP_PROG_UNSUCCESS                                                         */
#define OTP_F_LOTP_PROG_UNSUCCESS_R                                                        3
#define OTP_F_LOTP_PROG_UNSUCCESS_L                                                        3
#define OTP_F_LOTP_PROG_UNSUCCESS_SIZE                                                     1
#define OTP_F_LOTP_PROG_UNSUCCESS_MASK                                            0x00000008
#define OTP_F_LOTP_PROG_UNSUCCESS_RANGE                                                  3:3
#define OTP_F_LOTP_PROG_UNSUCCESS_INIT                                            0x00000000
#define OTP_F_LOTP_PROG_UNSUCCESS_RWMODE                                                 SRO
#define OTP_F_LOTP_PROG_UNSUCCESS_GET(x) \
    (((x) & OTP_F_LOTP_PROG_UNSUCCESS_MASK) >> \
            OTP_F_LOTP_PROG_UNSUCCESS_R) \
/* Field: OTP_F_LOTP_CPU_MODE_EN                                                            */
#define OTP_F_LOTP_CPU_MODE_EN_R                                                          15
#define OTP_F_LOTP_CPU_MODE_EN_L                                                          15
#define OTP_F_LOTP_CPU_MODE_EN_SIZE                                                        1
#define OTP_F_LOTP_CPU_MODE_EN_MASK                                               0x00008000
#define OTP_F_LOTP_CPU_MODE_EN_RANGE                                                   15:15
#define OTP_F_LOTP_CPU_MODE_EN_INIT                                               0x00000000
#define OTP_F_LOTP_CPU_MODE_EN_RWMODE                                                     RW
#define OTP_F_LOTP_CPU_MODE_EN_GET(x) \
    (((x) & OTP_F_LOTP_CPU_MODE_EN_MASK) >> \
            OTP_F_LOTP_CPU_MODE_EN_R) \
/* Field: OTP_F_LOTP_PROG_SUCCESS                                                           */
#define OTP_F_LOTP_PROG_SUCCESS_R                                                          2
#define OTP_F_LOTP_PROG_SUCCESS_L                                                          2
#define OTP_F_LOTP_PROG_SUCCESS_SIZE                                                       1
#define OTP_F_LOTP_PROG_SUCCESS_MASK                                              0x00000004
#define OTP_F_LOTP_PROG_SUCCESS_RANGE                                                    2:2
#define OTP_F_LOTP_PROG_SUCCESS_INIT                                              0x00000000
#define OTP_F_LOTP_PROG_SUCCESS_RWMODE                                                   SRO
#define OTP_F_LOTP_PROG_SUCCESS_GET(x) \
    (((x) & OTP_F_LOTP_PROG_SUCCESS_MASK) >> \
            OTP_F_LOTP_PROG_SUCCESS_R) \
/* Field: OTP_F_LOTP_ENABLE                                                                 */
#define OTP_F_LOTP_ENABLE_R                                                               12
#define OTP_F_LOTP_ENABLE_L                                                               12
#define OTP_F_LOTP_ENABLE_SIZE                                                             1
#define OTP_F_LOTP_ENABLE_MASK                                                    0x00001000
#define OTP_F_LOTP_ENABLE_RANGE                                                        12:12
#define OTP_F_LOTP_ENABLE_INIT                                                    0x00000000
#define OTP_F_LOTP_ENABLE_RWMODE                                                          RW
#define OTP_F_LOTP_ENABLE_GET(x) \
    (((x) & OTP_F_LOTP_ENABLE_MASK) >> \
            OTP_F_LOTP_ENABLE_R) \
/* Field: OTP_F_LOTP_CLK_DIV_RATIO                                                          */
#define OTP_F_LOTP_CLK_DIV_RATIO_R                                                        20
#define OTP_F_LOTP_CLK_DIV_RATIO_L                                                        21
#define OTP_F_LOTP_CLK_DIV_RATIO_SIZE                                                      2
#define OTP_F_LOTP_CLK_DIV_RATIO_MASK                                             0x00300000
#define OTP_F_LOTP_CLK_DIV_RATIO_RANGE                                                 21:20
#define OTP_F_LOTP_CLK_DIV_RATIO_INIT                                             0x00000002
#define OTP_F_LOTP_CLK_DIV_RATIO_RWMODE                                                   RW
#define OTP_F_LOTP_CLK_DIV_RATIO_GET(x) \
    (((x) & OTP_F_LOTP_CLK_DIV_RATIO_MASK) >> \
            OTP_F_LOTP_CLK_DIV_RATIO_R) \
/* Field: OTP_F_LOTP_CLK_DIV_EN                                                             */
#define OTP_F_LOTP_CLK_DIV_EN_R                                                           22
#define OTP_F_LOTP_CLK_DIV_EN_L                                                           22
#define OTP_F_LOTP_CLK_DIV_EN_SIZE                                                         1
#define OTP_F_LOTP_CLK_DIV_EN_MASK                                                0x00400000
#define OTP_F_LOTP_CLK_DIV_EN_RANGE                                                    22:22
#define OTP_F_LOTP_CLK_DIV_EN_INIT                                                0x00000001
#define OTP_F_LOTP_CLK_DIV_EN_RWMODE                                                      RW
#define OTP_F_LOTP_CLK_DIV_EN_GET(x) \
    (((x) & OTP_F_LOTP_CLK_DIV_EN_MASK) >> \
            OTP_F_LOTP_CLK_DIV_EN_R) \
/* Register: OTP_R_LOTP_ADDR                                                                */
#define OTP_R_LOTP_ADDR_MEMADDR                                                    0xC8008
#define OTP_R_LOTP_ADDR_SEL                                                              0x8
#define OTP_R_LOTP_ADDR_SIZE                                                              32
#define OTP_R_LOTP_ADDR_MASK                                                      0xffffffff
#define OTP_R_LOTP_ADDR_INIT                                                             0x0
#define OTP_R_LOTP_ADDR_RWMODE                                                            RW
/* Field: OTP_F_LOTP_BITSEL                                                                 */
#define OTP_F_LOTP_BITSEL_R                                                                0
#define OTP_F_LOTP_BITSEL_L                                                                4
#define OTP_F_LOTP_BITSEL_SIZE                                                             5
#define OTP_F_LOTP_BITSEL_MASK                                                    0x0000001f
#define OTP_F_LOTP_BITSEL_RANGE                                                          4:0
#define OTP_F_LOTP_BITSEL_INIT                                                    0x00000000
#define OTP_F_LOTP_BITSEL_RWMODE                                                          RW
#define OTP_F_LOTP_BITSEL_GET(x) \
    (((x) & OTP_F_LOTP_BITSEL_MASK) >> \
            OTP_F_LOTP_BITSEL_R) \
/* Field: OTP_F_LOTP_ADDR                                                                   */
#define OTP_F_LOTP_ADDR_R                                                                  5
#define OTP_F_LOTP_ADDR_L                                                                 15
#define OTP_F_LOTP_ADDR_SIZE                                                              11
#define OTP_F_LOTP_ADDR_MASK                                                      0x0000ffe0
#define OTP_F_LOTP_ADDR_RANGE                                                           15:5
#define OTP_F_LOTP_ADDR_INIT                                                      0x00000000
#define OTP_F_LOTP_ADDR_RWMODE                                                            RW
#define OTP_F_LOTP_ADDR_GET(x) \
    (((x) & OTP_F_LOTP_ADDR_MASK) >> \
            OTP_F_LOTP_ADDR_R) \
/* Register: OTP_R_LOTP_DEBUG                                                               */
#define OTP_R_LOTP_DEBUG_MEMADDR                                                   0xC8034
#define OTP_R_LOTP_DEBUG_SEL                                                            0x34
#define OTP_R_LOTP_DEBUG_SIZE                                                             32
#define OTP_R_LOTP_DEBUG_MASK                                                     0xffffffff
#define OTP_R_LOTP_DEBUG_INIT                                                            0x0
#define OTP_R_LOTP_DEBUG_RWMODE                                                          SRO
/* Field: OTP_F_LOTP_WRAPPER_WRP_BUSY                                                       */
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_R                                                     21
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_L                                                     21
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_SIZE                                                   1
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_MASK                                          0x00200000
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_RANGE                                              21:21
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_INIT                                          0x00000000
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_RWMODE                                               SRO
#define OTP_F_LOTP_WRAPPER_WRP_BUSY_GET(x) \
    (((x) & OTP_F_LOTP_WRAPPER_WRP_BUSY_MASK) >> \
            OTP_F_LOTP_WRAPPER_WRP_BUSY_R) \
/* Field: OTP_F_LOTP_WRAPPER_WRP_CUR_STATE                                                  */
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_R                                                16
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_L                                                20
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_SIZE                                              5
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_MASK                                     0x001f0000
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_RANGE                                         20:16
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_INIT                                     0x00000000
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_RWMODE                                          SRO
#define OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_GET(x) \
    (((x) & OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_MASK) >> \
            OTP_F_LOTP_WRAPPER_WRP_CUR_STATE_R) \
/* Field: OTP_F_LOTP_CONTROL_PROG_BIT_VLD                                                   */
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_R                                                 22
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_L                                                 22
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_SIZE                                               1
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_MASK                                      0x00400000
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_RANGE                                          22:22
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_INIT                                      0x00000000
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_RWMODE                                           SRO
#define OTP_F_LOTP_CONTROL_PROG_BIT_VLD_GET(x) \
    (((x) & OTP_F_LOTP_CONTROL_PROG_BIT_VLD_MASK) >> \
            OTP_F_LOTP_CONTROL_PROG_BIT_VLD_R) \
/* Field: OTP_F_LOTP_WRAPPER_WRP_ADDR                                                       */
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_R                                                      0
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_L                                                     15
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_SIZE                                                  16
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_MASK                                          0x0000ffff
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_RANGE                                               15:0
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_INIT                                          0x00000000
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_RWMODE                                               SRO
#define OTP_F_LOTP_WRAPPER_WRP_ADDR_GET(x) \
    (((x) & OTP_F_LOTP_WRAPPER_WRP_ADDR_MASK) >> \
            OTP_F_LOTP_WRAPPER_WRP_ADDR_R) \
/* Field: OTP_F_LOTP_RESERVED                                                               */
#define OTP_F_LOTP_RESERVED_R                                                             30
#define OTP_F_LOTP_RESERVED_L                                                             31
#define OTP_F_LOTP_RESERVED_SIZE                                                           2
#define OTP_F_LOTP_RESERVED_MASK                                                  0xc0000000
#define OTP_F_LOTP_RESERVED_RANGE                                                      31:30
#define OTP_F_LOTP_RESERVED_INIT                                                  0x00000000
#define OTP_F_LOTP_RESERVED_RWMODE                                                       SRO
#define OTP_F_LOTP_RESERVED_GET(x) \
    (((x) & OTP_F_LOTP_RESERVED_MASK) >> \
            OTP_F_LOTP_RESERVED_R) \
/* Field: OTP_F_LOTP_CELLS_BUSY                                                             */
#define OTP_F_LOTP_CELLS_BUSY_R                                                           27
#define OTP_F_LOTP_CELLS_BUSY_L                                                           28
#define OTP_F_LOTP_CELLS_BUSY_SIZE                                                         2
#define OTP_F_LOTP_CELLS_BUSY_MASK                                                0x18000000
#define OTP_F_LOTP_CELLS_BUSY_RANGE                                                    28:27
#define OTP_F_LOTP_CELLS_BUSY_INIT                                                0x00000000
#define OTP_F_LOTP_CELLS_BUSY_RWMODE                                                     SRO
#define OTP_F_LOTP_CELLS_BUSY_GET(x) \
    (((x) & OTP_F_LOTP_CELLS_BUSY_MASK) >> \
            OTP_F_LOTP_CELLS_BUSY_R) \
/* Field: OTP_F_LOTP_CLK                                                                    */
#define OTP_F_LOTP_CLK_R                                                                  29
#define OTP_F_LOTP_CLK_L                                                                  29
#define OTP_F_LOTP_CLK_SIZE                                                                1
#define OTP_F_LOTP_CLK_MASK                                                       0x20000000
#define OTP_F_LOTP_CLK_RANGE                                                           29:29
#define OTP_F_LOTP_CLK_INIT                                                       0x00000000
#define OTP_F_LOTP_CLK_RWMODE                                                            SRO
#define OTP_F_LOTP_CLK_GET(x) \
    (((x) & OTP_F_LOTP_CLK_MASK) >> \
            OTP_F_LOTP_CLK_R) \
/* Field: OTP_F_LOTP_CONTROL_JOTP_STATE                                                     */
#define OTP_F_LOTP_CONTROL_JOTP_STATE_R                                                   23
#define OTP_F_LOTP_CONTROL_JOTP_STATE_L                                                   26
#define OTP_F_LOTP_CONTROL_JOTP_STATE_SIZE                                                 4
#define OTP_F_LOTP_CONTROL_JOTP_STATE_MASK                                        0x07800000
#define OTP_F_LOTP_CONTROL_JOTP_STATE_RANGE                                            26:23
#define OTP_F_LOTP_CONTROL_JOTP_STATE_INIT                                        0x00000000
#define OTP_F_LOTP_CONTROL_JOTP_STATE_RWMODE                                             SRO
#define OTP_F_LOTP_CONTROL_JOTP_STATE_GET(x) \
    (((x) & OTP_F_LOTP_CONTROL_JOTP_STATE_MASK) >> \
            OTP_F_LOTP_CONTROL_JOTP_STATE_R) \
/* Register: OTP_R_LOTP_RDATA                                                               */
#define OTP_R_LOTP_RDATA_MEMADDR                                                   0xC8014
#define OTP_R_LOTP_RDATA_SEL                                                            0x14
#define OTP_R_LOTP_RDATA_SIZE                                                             32
#define OTP_R_LOTP_RDATA_MASK                                                     0xffffffff
#define OTP_R_LOTP_RDATA_INIT                                                            0x0
#define OTP_R_LOTP_RDATA_RWMODE                                                          SRO
/* Field: OTP_F_LOTP_RDATA_FAIL                                                             */
#define OTP_F_LOTP_RDATA_FAIL_R                                                           29
#define OTP_F_LOTP_RDATA_FAIL_L                                                           31
#define OTP_F_LOTP_RDATA_FAIL_SIZE                                                         3
#define OTP_F_LOTP_RDATA_FAIL_MASK                                                0xe0000000
#define OTP_F_LOTP_RDATA_FAIL_RANGE                                                    31:29
#define OTP_F_LOTP_RDATA_FAIL_INIT                                                0x00000000
#define OTP_F_LOTP_RDATA_FAIL_RWMODE                                                     SRO
#define OTP_F_LOTP_RDATA_FAIL_GET(x) \
    (((x) & OTP_F_LOTP_RDATA_FAIL_MASK) >> \
            OTP_F_LOTP_RDATA_FAIL_R) \
/* Field: OTP_F_LOTP_RDATA_DOUT                                                             */
#define OTP_F_LOTP_RDATA_DOUT_R                                                            0
#define OTP_F_LOTP_RDATA_DOUT_L                                                           23
#define OTP_F_LOTP_RDATA_DOUT_SIZE                                                        24
#define OTP_F_LOTP_RDATA_DOUT_MASK                                                0x00ffffff
#define OTP_F_LOTP_RDATA_DOUT_RANGE                                                     23:0
#define OTP_F_LOTP_RDATA_DOUT_INIT                                                0x00000000
#define OTP_F_LOTP_RDATA_DOUT_RWMODE                                                     SRO
#define OTP_F_LOTP_RDATA_DOUT_GET(x) \
    (((x) & OTP_F_LOTP_RDATA_DOUT_MASK) >> \
            OTP_F_LOTP_RDATA_DOUT_R) \
/* Field: OTP_F_LOTP_RDATA_ECC                                                              */
#define OTP_F_LOTP_RDATA_ECC_R                                                            24
#define OTP_F_LOTP_RDATA_ECC_L                                                            28
#define OTP_F_LOTP_RDATA_ECC_SIZE                                                          5
#define OTP_F_LOTP_RDATA_ECC_MASK                                                 0x1f000000
#define OTP_F_LOTP_RDATA_ECC_RANGE                                                     28:24
#define OTP_F_LOTP_RDATA_ECC_INIT                                                 0x00000000
#define OTP_F_LOTP_RDATA_ECC_RWMODE                                                      SRO
#define OTP_F_LOTP_RDATA_ECC_GET(x) \
    (((x) & OTP_F_LOTP_RDATA_ECC_MASK) >> \
            OTP_F_LOTP_RDATA_ECC_R) \
/* Register: OTP_R_LOTP_CTRL                                                                */
#define OTP_R_LOTP_CTRL_MEMADDR                                                    0xC800c
#define OTP_R_LOTP_CTRL_SEL                                                              0xc
#define OTP_R_LOTP_CTRL_SIZE                                                              32
#define OTP_R_LOTP_CTRL_MASK                                                      0xffffffff
#define OTP_R_LOTP_CTRL_INIT                                                        0x130800
#define OTP_R_LOTP_CTRL_RWMODE                                                            RW
/* Field: OTP_F_LOTP_CTRL_ACCESS_MODE                                                       */
#define OTP_F_LOTP_CTRL_ACCESS_MODE_R                                                     19
#define OTP_F_LOTP_CTRL_ACCESS_MODE_L                                                     20
#define OTP_F_LOTP_CTRL_ACCESS_MODE_SIZE                                                   2
#define OTP_F_LOTP_CTRL_ACCESS_MODE_MASK                                          0x00180000
#define OTP_F_LOTP_CTRL_ACCESS_MODE_RANGE                                              20:19
#define OTP_F_LOTP_CTRL_ACCESS_MODE_INIT                                          0x00000002
#define OTP_F_LOTP_CTRL_ACCESS_MODE_RWMODE                                                RW
#define OTP_F_LOTP_CTRL_ACCESS_MODE_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_ACCESS_MODE_MASK) >> \
            OTP_F_LOTP_CTRL_ACCESS_MODE_R) \
/* Field: OTP_F_LOTP_CTRL_SADBYP                                                            */
#define OTP_F_LOTP_CTRL_SADBYP_R                                                          13
#define OTP_F_LOTP_CTRL_SADBYP_L                                                          13
#define OTP_F_LOTP_CTRL_SADBYP_SIZE                                                        1
#define OTP_F_LOTP_CTRL_SADBYP_MASK                                               0x00002000
#define OTP_F_LOTP_CTRL_SADBYP_RANGE                                                   13:13
#define OTP_F_LOTP_CTRL_SADBYP_INIT                                               0x00000000
#define OTP_F_LOTP_CTRL_SADBYP_RWMODE                                                     RW
#define OTP_F_LOTP_CTRL_SADBYP_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_SADBYP_MASK) >> \
            OTP_F_LOTP_CTRL_SADBYP_R) \
/* Field: OTP_F_LOTP_CTRL_ECC_WREN                                                          */
#define OTP_F_LOTP_CTRL_ECC_WREN_R                                                        24
#define OTP_F_LOTP_CTRL_ECC_WREN_L                                                        24
#define OTP_F_LOTP_CTRL_ECC_WREN_SIZE                                                      1
#define OTP_F_LOTP_CTRL_ECC_WREN_MASK                                             0x01000000
#define OTP_F_LOTP_CTRL_ECC_WREN_RANGE                                                 24:24
#define OTP_F_LOTP_CTRL_ECC_WREN_INIT                                             0x00000000
#define OTP_F_LOTP_CTRL_ECC_WREN_RWMODE                                                   RW
#define OTP_F_LOTP_CTRL_ECC_WREN_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_ECC_WREN_MASK) >> \
            OTP_F_LOTP_CTRL_ECC_WREN_R) \
/* Field: OTP_F_LOTP_CTRL_TM                                                                */
#define OTP_F_LOTP_CTRL_TM_R                                                              14
#define OTP_F_LOTP_CTRL_TM_L                                                              16
#define OTP_F_LOTP_CTRL_TM_SIZE                                                            3
#define OTP_F_LOTP_CTRL_TM_MASK                                                   0x0001c000
#define OTP_F_LOTP_CTRL_TM_RANGE                                                       16:14
#define OTP_F_LOTP_CTRL_TM_INIT                                                   0x00000004
#define OTP_F_LOTP_CTRL_TM_RWMODE                                                         RW
#define OTP_F_LOTP_CTRL_TM_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_TM_MASK) >> \
            OTP_F_LOTP_CTRL_TM_R) \
/* Field: OTP_F_LOTP_CTRL_PCOUNT                                                            */
#define OTP_F_LOTP_CTRL_PCOUNT_R                                                           9
#define OTP_F_LOTP_CTRL_PCOUNT_L                                                          11
#define OTP_F_LOTP_CTRL_PCOUNT_SIZE                                                        3
#define OTP_F_LOTP_CTRL_PCOUNT_MASK                                               0x00000e00
#define OTP_F_LOTP_CTRL_PCOUNT_RANGE                                                    11:9
#define OTP_F_LOTP_CTRL_PCOUNT_INIT                                               0x00000004
#define OTP_F_LOTP_CTRL_PCOUNT_RWMODE                                                     RW
#define OTP_F_LOTP_CTRL_PCOUNT_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_PCOUNT_MASK) >> \
            OTP_F_LOTP_CTRL_PCOUNT_R) \
/* Field: OTP_F_LOTP_CTRL_DEBUG_MODE                                                        */
#define OTP_F_LOTP_CTRL_DEBUG_MODE_R                                                      18
#define OTP_F_LOTP_CTRL_DEBUG_MODE_L                                                      18
#define OTP_F_LOTP_CTRL_DEBUG_MODE_SIZE                                                    1
#define OTP_F_LOTP_CTRL_DEBUG_MODE_MASK                                           0x00040000
#define OTP_F_LOTP_CTRL_DEBUG_MODE_RANGE                                               18:18
#define OTP_F_LOTP_CTRL_DEBUG_MODE_INIT                                           0x00000000
#define OTP_F_LOTP_CTRL_DEBUG_MODE_RWMODE                                                 RW
#define OTP_F_LOTP_CTRL_DEBUG_MODE_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_DEBUG_MODE_MASK) >> \
            OTP_F_LOTP_CTRL_DEBUG_MODE_R) \
/* Field: OTP_F_LOTP_CTRL_OTP_CMD                                                           */
#define OTP_F_LOTP_CTRL_OTP_CMD_R                                                          0
#define OTP_F_LOTP_CTRL_OTP_CMD_L                                                          3
#define OTP_F_LOTP_CTRL_OTP_CMD_SIZE                                                       4
#define OTP_F_LOTP_CTRL_OTP_CMD_MASK                                              0x0000000f
#define OTP_F_LOTP_CTRL_OTP_CMD_RANGE                                                    3:0
#define OTP_F_LOTP_CTRL_OTP_CMD_INIT                                              0x00000000
#define OTP_F_LOTP_CTRL_OTP_CMD_RWMODE                                                    RW
#define OTP_F_LOTP_CTRL_OTP_CMD_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_OTP_CMD_MASK) >> \
            OTP_F_LOTP_CTRL_OTP_CMD_R) \
/* Field: OTP_F_LOTP_CTRL_VSEL                                                              */
#define OTP_F_LOTP_CTRL_VSEL_R                                                             5
#define OTP_F_LOTP_CTRL_VSEL_L                                                             8
#define OTP_F_LOTP_CTRL_VSEL_SIZE                                                          4
#define OTP_F_LOTP_CTRL_VSEL_MASK                                                 0x000001e0
#define OTP_F_LOTP_CTRL_VSEL_RANGE                                                       8:5
#define OTP_F_LOTP_CTRL_VSEL_INIT                                                 0x00000000
#define OTP_F_LOTP_CTRL_VSEL_RWMODE                                                       RW
#define OTP_F_LOTP_CTRL_VSEL_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_VSEL_MASK) >> \
            OTP_F_LOTP_CTRL_VSEL_R) \
/* Field: OTP_F_LOTP_CTRL_PROGSEL                                                           */
#define OTP_F_LOTP_CTRL_PROGSEL_R                                                          4
#define OTP_F_LOTP_CTRL_PROGSEL_L                                                          4
#define OTP_F_LOTP_CTRL_PROGSEL_SIZE                                                       1
#define OTP_F_LOTP_CTRL_PROGSEL_MASK                                              0x00000010
#define OTP_F_LOTP_CTRL_PROGSEL_RANGE                                                    4:4
#define OTP_F_LOTP_CTRL_PROGSEL_INIT                                              0x00000000
#define OTP_F_LOTP_CTRL_PROGSEL_RWMODE                                                    RW
#define OTP_F_LOTP_CTRL_PROGSEL_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_PROGSEL_MASK) >> \
            OTP_F_LOTP_CTRL_PROGSEL_R) \
/* Field: OTP_F_LOTP_CTRL_BURST_STAT_SEL                                                    */
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_R                                                  21
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_L                                                  21
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_SIZE                                                1
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_MASK                                       0x00200000
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_RANGE                                           21:21
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_INIT                                       0x00000000
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_RWMODE                                             RW
#define OTP_F_LOTP_CTRL_BURST_STAT_SEL_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_BURST_STAT_SEL_MASK) >> \
            OTP_F_LOTP_CTRL_BURST_STAT_SEL_R) \
/* Field: OTP_F_LOTP_CTRL_PBYP                                                              */
#define OTP_F_LOTP_CTRL_PBYP_R                                                            12
#define OTP_F_LOTP_CTRL_PBYP_L                                                            12
#define OTP_F_LOTP_CTRL_PBYP_SIZE                                                          1
#define OTP_F_LOTP_CTRL_PBYP_MASK                                                 0x00001000
#define OTP_F_LOTP_CTRL_PBYP_RANGE                                                     12:12
#define OTP_F_LOTP_CTRL_PBYP_INIT                                                 0x00000000
#define OTP_F_LOTP_CTRL_PBYP_RWMODE                                                       RW
#define OTP_F_LOTP_CTRL_PBYP_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_PBYP_MASK) >> \
            OTP_F_LOTP_CTRL_PBYP_R) \
/* Field: OTP_F_LOTP_CTRL_ENECC                                                             */
#define OTP_F_LOTP_CTRL_ENECC_R                                                           25
#define OTP_F_LOTP_CTRL_ENECC_L                                                           25
#define OTP_F_LOTP_CTRL_ENECC_SIZE                                                         1
#define OTP_F_LOTP_CTRL_ENECC_MASK                                                0x02000000
#define OTP_F_LOTP_CTRL_ENECC_RANGE                                                    25:25
#define OTP_F_LOTP_CTRL_ENECC_INIT                                                0x00000000
#define OTP_F_LOTP_CTRL_ENECC_RWMODE                                                      RW
#define OTP_F_LOTP_CTRL_ENECC_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_ENECC_MASK) >> \
            OTP_F_LOTP_CTRL_ENECC_R) \
/* Field: OTP_F_LOTP_CTRL_CONT_ON_FAIL                                                      */
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_R                                                    17
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_L                                                    17
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_SIZE                                                  1
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_MASK                                         0x00020000
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_RANGE                                             17:17
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_INIT                                         0x00000001
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_RWMODE                                               RW
#define OTP_F_LOTP_CTRL_CONT_ON_FAIL_GET(x) \
    (((x) & OTP_F_LOTP_CTRL_CONT_ON_FAIL_MASK) >> \
            OTP_F_LOTP_CTRL_CONT_ON_FAIL_R) \
/* Register: OTP_R_LOTP_BAD_ADDR                                                            */
#define OTP_R_LOTP_BAD_ADDR_MEMADDR                                                0xC801c
#define OTP_R_LOTP_BAD_ADDR_SEL                                                         0x1c
#define OTP_R_LOTP_BAD_ADDR_SIZE                                                          32
#define OTP_R_LOTP_BAD_ADDR_MASK                                                  0xffffffff
#define OTP_R_LOTP_BAD_ADDR_INIT                                                         0x0
#define OTP_R_LOTP_BAD_ADDR_RWMODE                                                       SRO
/* Field: OTP_F_LOTP_BAD_ADDR                                                               */
#define OTP_F_LOTP_BAD_ADDR_R                                                              0
#define OTP_F_LOTP_BAD_ADDR_L                                                              7
#define OTP_F_LOTP_BAD_ADDR_SIZE                                                           8
#define OTP_F_LOTP_BAD_ADDR_MASK                                                  0x000000ff
#define OTP_F_LOTP_BAD_ADDR_RANGE                                                        7:0
#define OTP_F_LOTP_BAD_ADDR_INIT                                                  0x00000000
#define OTP_F_LOTP_BAD_ADDR_RWMODE                                                       SRO
#define OTP_F_LOTP_BAD_ADDR_GET(x) \
    (((x) & OTP_F_LOTP_BAD_ADDR_MASK) >> \
            OTP_F_LOTP_BAD_ADDR_R) 

/* Register: OTP_R_LOTP_SECURE0
 * Reset only by Power ON. Will disable all CPU access to OTP.
 */
#define OTP_R_LOTP_SECURE0_MEMADDR                                                   0xB0f00
#define OTP_R_LOTP_SECURE0_SEL                                                         0xf00
#define OTP_R_LOTP_SECURE0_SIZE                                                           32
#define OTP_R_LOTP_SECURE0_MASK                                                   0x00000001
#define OTP_R_LOTP_SECURE0_INIT                                                          0x0
#define OTP_R_LOTP_SECURE0_RWMODE                                                         RW

/* Register: OTP_R_LOTP_SECURE1
 * Reset only by Power ON. Will disable 32 rows of the OTP. Each bit corresponds to a row of
 * the OTP
 */
#define OTP_R_LOTP_SECURE1_MEMADDR                                                   0xB0f04
#define OTP_R_LOTP_SECURE1_SEL                                                         0xf04
#define OTP_R_LOTP_SECURE1_SIZE                                                           32
#define OTP_R_LOTP_SECURE1_MASK                                                   0xffffffff
#define OTP_R_LOTP_SECURE1_INIT                                                          0x0
#define OTP_R_LOTP_SECURE1_RWMODE                                                         RW

#endif /* OTP_CPUAPI_H */

