/*
 * TomTom U-Boot meta information header
 *
 * Copyright (C) 2010 TomTom International B.V.
 * Author: Martin Jackson <martin.jackson@tomtom.com>
 * 
 ************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 ************************************************************************
 */

#ifndef __TT_HEADER_H
#define __TT_HEADER_H

#ifndef __ASSEMBLER__
# include <asm-arm/byteorder.h>
# ifdef USE_HOSTCC
#  include <stdint.h>
# else
#  include <linux/stddef.h>
# endif
# include <linux/types.h>
#endif

/*
 * Header revision history
 * =======================
 *
 * v1.1 2010/02/10 - Martin Jackson <martin.jackson@tomtom.com>
 * Initial revision
 */


/*
 * TT_HEADER Version
 */

/* Major version - uprev only when compatibility breaks */
#define TT_HEADER_MAJOR		1
/* Minor version - uprev when format changes but remains compatible */
#define TT_HEADER_MINOR		1


/*
 * TT_HEADER Format structure
 */

#define TT_HEADER_OFFSET	64	/* Offset in u-boot.bin */
#define TT_HEADER_LEN		64	/* Length of header struct */
#define TT_HEADER_MAGIC		"TTBH"	/* Header invalid if absent */

/**
 * The 'brick' number is used to break compatibility of released devices
 * with bootloader releases. If the brick number saved in OTP < current
 * brick number, U-Boot refuses to run.
 **/
#define TT_HEADER_BRICK_CURRENT	0	/* ubootctl uses this value,
					 * uboot itself checks against
					 * value from header
					 */

#ifndef __ASSEMBLER__
struct tt_header {
	uint32_t magic;		/* "TTBH" */
	uint16_t len;		/* Length of the entire header */
	uint8_t ver_major;	/* Major version of header */
	uint8_t ver_minor;	/* Minor version of header */
	uint32_t uboot_version;	/* P4 CL of U-Boot in this binary */

#define TT_HEADER_PLATLEN	20 
	/* Platform name - WARNING NOT NECESSARILY NUL-TERMINATED */
	char platform[TT_HEADER_PLATLEN];
	uint32_t brick;		/* 'brick' number, see above */

	/* Add here. Rembmer we are limited by TT_HEADER_LEN */

} __attribute__((__packed__));

#define MKHDR(x) ((struct tt_header*)addr)

/* Read the struct in machine independent way */
static inline void __read_tt_header(void *addr, struct tt_header *hdr)
{
	hdr->magic = __le32_to_cpu(MKHDR(addr)->magic);
	hdr->len = __le16_to_cpu(MKHDR(addr)->len);
	hdr->ver_major = MKHDR(addr)->ver_major;
	hdr->ver_minor = MKHDR(addr)->ver_minor;
	hdr->uboot_version =
		__le32_to_cpu(MKHDR(addr)->uboot_version);
	strncpy(hdr->platform, MKHDR(addr)->platform,
		TT_HEADER_PLATLEN);
	hdr->brick = __le32_to_cpu(MKHDR(addr)->brick);
}

/* Write the struct in machine independent way */
static inline void __write_tt_header(struct tt_header *hdr, void *addr)
{
	MKHDR(addr)->magic = __cpu_to_le32(hdr->magic);
	MKHDR(addr)->len = __cpu_to_le16(hdr->len);
	MKHDR(addr)->ver_major = hdr->ver_major;
	MKHDR(addr)->ver_minor = hdr->ver_minor;
	MKHDR(addr)->uboot_version =
		__cpu_to_le32(hdr->uboot_version);
	strncpy(MKHDR(addr)->platform, hdr->platform,
		TT_HEADER_PLATLEN);
	MKHDR(addr)->brick = __cpu_to_le32(hdr->brick);
}

static void inline __dump_tt_header(void *raw)
{
	struct tt_header hdr;

#ifdef USE_HOSTCC
	FILE *f = popen("hexdump -C", "w");
	if (f == NULL) {
		warn("Could not run hexdump(1)");
		return;
	}
#endif

	__read_tt_header(raw, &hdr);
	printf("==== Dumping header ====\n");
#ifdef USE_HOSTCC
	fwrite(raw, sizeof(struct tt_header), 1, f);
	pclose(f);
#else
/* TODO */
#endif
	printf("========================\n");
	printf("magic         = %#x [%.*s]\n", hdr.magic,
	       (int) sizeof(uint32_t), (char *) &hdr.magic);
	printf("len           = %u\n", hdr.len);
	printf("ver_major     = %u\n", hdr.ver_major);
	printf("ver_minor     = %u\n", hdr.ver_minor);
	printf("uboot_version = %u\n", hdr.uboot_version);
	printf("platform      = [%.*s]\n", TT_HEADER_PLATLEN,
	       hdr.platform);
	printf("brick         = %u\n", hdr.brick);
}

extern int check_bricknum(void);

#endif /* __ASSEMBLER__ */


#endif /* __TT_HEADER_H */

